#include "gamesa.h"
#include <stdio.h>
#include <stdlib.h>
#define MAXN 100000
#define P 400000

static int hrany[2 * MAXN][2], uhlopricky[MAXN][2], id_hrany[P];
static int obarvena[2 * MAXN], N, M = 0;

static unsigned hash (unsigned a, unsigned b) {
  return a * N + b;
}

static void pridej_hranu(int a, int b) {
  if (a > b) {
    int tmp = a;
    a = b;
    b = tmp;
  }

  unsigned h = hash(a, b) % P;
  while (id_hrany[h]) h = (h + 1) % P;

  hrany[M][0] = a;
  hrany[M][1] = b;
  id_hrany[h] = ++M;
}

static int najdi_hranu (int a, int b) {
  if (a > b) {
    int tmp = a;
    a = b;
    b = tmp;
  }

  unsigned h = hash(a, b) % P;
  while (id_hrany[h]) {
    if (hrany[id_hrany[h] - 1][0] == a && hrany[id_hrany[h] - 1][1] == b) return id_hrany[h] - 1;
    h = (h + 1) % P;
  }

  return -1;
}

static int obarveny_trojuhelnik (int h) {
  for (int i = 0; i < N; i++) {
    int h1 = najdi_hranu(hrany[h][0], i), h2 = najdi_hranu(hrany[h][1], i);
    if(h1 != -1 && h2 != -1 && obarvena[h1] && obarvena[h2]) return 1;
  }
  return 0;
}

int main (int argc, char *argv[]) {
  int tah[2], h;

  if (argc != 2) {
    fprintf (stderr, "Ocekavam soubor s popisem hraciho planu jako argument.\n");
    return 0;
  }

  FILE *plan = fopen (argv[1], "r");
  if (!plan) {
    fprintf (stderr, "Popis hraciho planu nelze otevrit.\n");
    return 0;
  }

  fscanf (plan, "%d", &N);

  for (int i = 0; i < N ; i++) pridej_hranu (i + 1, (i + 1) % N + 1);
  for (int i = 0; i < N - 3; i++) {
    fscanf (plan, "%d%d", &uhlopricky[i][0], &uhlopricky[i][1]);
    pridej_hranu (uhlopricky[i][0], uhlopricky[i][1]);
  }
  fclose (plan);

  herni_plan (N, uhlopricky);

  while (1) {
    tah_honzika (tah);
    fprintf (stderr, "Tah Honzika: %d %d\n", tah[0], tah[1]);

    h = najdi_hranu (tah[0], tah[1]);
    if (h == -1) {
      fprintf (stderr, "Neexistujici hrana.\n");
      return 1;
    }
    if (obarvena[h]) {
      fprintf (stderr, "Jiz obarvena hrana.\n");
      return 1;
    }
    obarvena[h] = 1;

    if (obarveny_trojuhelnik (h)) {
      fprintf (stderr, "Honzik vyhral.\n");
      return 0;
    }

    fprintf (stderr, "Tah Marenky?\n");
    while(1) {
      scanf ("%d%d", &tah[0], &tah[1]);
      if (tah[0] == 0 && tah[1] == 0) { // nahodny tah
        do { h = rand() % M; } while(obarvena[h]);
        tah[0] = hrany[h][0];
        tah[1] = hrany[h][1];
        fprintf (stderr, "Nahodny tah Marenky: %d %d\n", tah[0], tah[1]);
      }
      else h = najdi_hranu (tah[0], tah[1]);
      
      if (h == -1) {
        fprintf (stderr, "Hrana neexistuje; zadejte jinou.\n");
        continue;
      }
      
      if (obarvena[h]) {
        fprintf (stderr, "Jiz obarvena hrana; zadejte jinou.\n");
        continue;
      }
      break;
    }
    obarvena[h] = 1;
    
    if (obarveny_trojuhelnik (h)) {
      fprintf (stderr, "Marenka vyhrala.\n");
      return 0;
    }
    tah_marenky (tah);
  }
  
  return 0;
}
