#include <unordered_map>
#include <unordered_set>
#include <set>
#include <vector>
#include <cstdio>
#include <cstdlib>

#ifdef CINTF
extern "C" {
#endif
#include "gamesa.h"
#ifdef CINTF
}
#endif

using namespace std;
#define MAXN 100000

static FILE *log;

static void log_err(const char *msg)
{
  fprintf(log, "<<error>> %s\n", msg);
  exit(0);
}

static void log_ok(const char *msg)
{
  fprintf(log, "<<ok>> %s\n", msg);
  exit(0);
}

struct hrana
{
  unsigned vs[2];

  hrana(void)
    {
      vs[0] = vs[1] = 0;
    }

  hrana(int v1, int v2)
    {
      if (v1 < v2)
	{
	  vs[0] = v1;
	  vs[1] = v2;
	}
      else
	{
	  vs[0] = v2;
	  vs[1] = v1;
	}
    }

  bool operator==(const hrana &s) const
    {
      return vs[0] == s.vs[0] && vs[1] == s.vs[1];
    }
};

static unsigned n;
namespace std {
template <> class hash< hrana > : public unary_function< hrana, size_t >
{
  public:
  size_t operator()(const hrana &h) const
    {
      return h.vs[0] * n + h.vs[1];
    }
};}

struct cesta
{
  unsigned od, len;

  cesta(void)
    {
      od = 0;
      len = 0;
    }

  cesta(int iod, int ilen)
    {
      od = iod;
      len = ilen;
    }

  bool operator==(const cesta &s) const
    {
      return od == s.od && len == s.len;
    }
};
inline bool operator< (const cesta& lhs, const cesta& rhs)
{
  return lhs.od + lhs.len - 1 < rhs.od;
}
namespace std {
template <> class hash< cesta > : public unary_function< cesta, size_t >
{
  public:
  size_t operator()(const cesta &c) const
    {
      return c.od * n + c.len;
    }
};}

#define MAX_NIMBER 8
#define SPORADIC ((int) (sizeof (nimbers) / sizeof (int)))
static const int nimbers[] =
{
  0,1,2,3,1,4,3,2,1,4,2,6,
  4,1,2,7,1,4,3,2,1,4,6,7,
  4,1,2,8,5,4,7,2,1,8,6,7,
  4,1,2,3,1,4,7,2,1,8,2,7,
  4,1,2,8,1,4,7,2,1,4,2,7,
  4,1,2,8,1,4,7,2,1,8,6,7};

static const int pernim[] =
{
  4,1,2,8,1,4,7,2,1,8,2,7
};

  /* Nezporn sla odpovdaj vrcholm v cest (od 0 do n - 3).
     Zporn odpovdaj dvojici vrchol -i - 1 a -i (kde i je od -1 do -(n-3)).  */
static unordered_map<hrana,int> vceste;
static unordered_map<hrana,bool> obarveny;
static vector<hrana> vrcholy;
static vector<int> obhran;
static vector<hrana> dvojice;

static set<cesta> zbyvajici_cesty;
static unordered_set<cesta> cesty_s_nim[MAX_NIMBER + 1];
static int animber;

static int
get_nimber (int len)
{
  if (len < SPORADIC)
    return nimbers[len];
  else
    return pernim[len % 12];
}

static unsigned
pred (unsigned k)
{
  return k == 1 ? n : k - 1;
}

static unsigned
nasl (unsigned k)
{
  return k == n ? 1 : k + 1;
}

static void
inicializace (int uhlopricky[][2])
{
  unsigned i, cv;
  int h, d;
  int deg[n + 1];

  vrcholy.resize (n);
  obhran.resize (n);
  dvojice.resize (n);

  for (i = 0; i < n - 2; i++)
    obhran[i] = 0;

  for (i = 1; i <= n; i++)
    deg[i] = 0;

  for (i = 0; i < n - 3; i++)
    {
      deg[uhlopricky[i][0]]++;
      deg[uhlopricky[i][1]]++;
      vceste[hrana (uhlopricky[i][0], uhlopricky[i][1])] = 0;
    };
  for (i = 1; deg[i] > 0; i++)
    continue;

  h = nasl (i);
  d = pred (i);
  vceste[hrana (i, h)] = 0;
  vceste[hrana (i, d)] = 0;
  dvojice[0] = hrana (i, h);
  vrcholy[0] = hrana (i, d);

  for (cv = 1; cv < n - 3; cv++)
    {
      int nh = nasl (h), nd = pred (d);
	  
      vceste[hrana (d, h)] = -cv;
      dvojice[cv] = hrana (d, h);
      if (vceste.count (hrana (d, nh)))
	{
	  vceste[hrana (h, nh)] = cv;
	  vrcholy[cv] = hrana (h, nh);
	  h = nh;
	}
      else
	{
	  vceste[hrana (d, nd)] = cv;
	  vrcholy[cv] = hrana (d, nd);
	  d = nd;
	}
    }

  if (n > 3)
    {
      i = nasl (h);
      vceste[hrana (d, h)] = -cv;
      dvojice[cv] = hrana (d, h);
      vceste[hrana (h, i)] = cv;
      vceste[hrana (d, i)] = cv;
      vrcholy[cv] = hrana (h, i);
      dvojice[cv + 1] = hrana (d, i);
    }
  else
    {
      vceste[hrana (d, h)] = 0;
      dvojice[1] = hrana (d, h);
    }

  for (i = 0; i < n - 2; i++)
    obarveny[vrcholy[i]] = false;
  for (i = 0; i < n - 1; i++)
    obarveny[dvojice[i]] = false;

  zbyvajici_cesty.insert (cesta (0, n - 2));
  animber = get_nimber (n - 2);
  cesty_s_nim[animber].insert (cesta (0, n - 2));
}

static bool
test_honzik_vyhral (int trojuhelnik)
{
  if (obhran[trojuhelnik] != 2)
    return false;
  log_ok ("Honzik vyhral.");
  return true;
}

static void
zvitezit (int trojuhelnik)
{
  hrana h[3];
  int i;

  h[0] = vrcholy[trojuhelnik];
  h[1] = dvojice[trojuhelnik];
  h[2] = dvojice[trojuhelnik + 1];
  for (i = 0; obarveny[h[i]]; i++)
    continue;

  fprintf (log, "M: %d %d\n", h[i].vs[0], h[i].vs[1]);
  log_err ("Marenka vyhrala.");
}

static void
najdi_tah (int len, int tonim, int *frem, int *nrem)
{
  int fr;

  for (fr = 0; ; fr++)
    {
      int amove;

      amove = get_nimber (fr) ^ get_nimber (len - fr - 1);
      if (amove == tonim)
	{
	  *frem = fr;
	  *nrem = 1;
	  return;
	}
      amove = get_nimber (fr) ^ get_nimber (len - fr - 2);
      if (amove == tonim)
	{
	  *frem = fr;
	  *nrem = 2;
	  return;
	}
    }
}

static void
zvol_tah_marenky (int tah[2])
{
  int mf, mt, frem, nrem;
  cesta ac;
  hrana h;

  // fprintf (log, "Nimber %d.\n", animber);

  if (animber == 0)
    {
      for (mf = MAX_NIMBER; mf > 0 && cesty_s_nim[mf].empty (); mf--)
	continue;

      if (mf == 0)
	{
	  unordered_map<hrana,bool>::iterator ah = obarveny.begin ();

	  while (ah->second)
	    ++ah;

	  h = ah->first;
	  goto end;
	}

      mt = mf - 1;
    }
  else
    {
      for (mf = 1; (mt = mf ^ animber) >= mf || cesty_s_nim[mf].empty (); mf++)
	continue;
    }

  // fprintf (log, "Moving from %d to %d.\n", mf, mt);
  ac = *(cesty_s_nim[mf].begin());
  najdi_tah (ac.len, mt, &frem, &nrem);

  if (nrem == 1)
    h = vrcholy[ac.od + frem];
  else
    h = dvojice[ac.od + frem + 1];

end:
  tah[0] = h.vs[0];
  tah[1] = h.vs[1];
}

static int uhlopricky[MAXN][2];
static int should_play = -1;

static void
pridej_cestu (int od, int len)
{
  if (len == 0)
    return;

  int nim = get_nimber (len);

  zbyvajici_cesty.insert (cesta (od, len));
  cesty_s_nim[nim].insert (cesta (od, len));
  animber ^= nim;
}

static void
nahodny_tah (int tah[2])
{
  hrana h;

  do
    {
      unsigned k = rand () % (2 * n - 3);

      if (k < n - 1)
	h = dvojice[k];
      else
	h = vrcholy[k - n + 1];
    }
  while (obarveny[h]);

  tah[0] = h.vs[0];
  tah[1] = h.vs[1];
}

static void
zvys_obarvenost (int tr)
{
  int nob = ++obhran[tr];
  if (nob == 1)
    {
      set<cesta>::iterator acp = zbyvajici_cesty.find (cesta (tr, 1));
      cesta ac = *acp;
      int nim = get_nimber (ac.len);
      zbyvajici_cesty.erase (acp);
      cesty_s_nim[nim].erase (ac);

      animber ^= nim;

      pridej_cestu (ac.od, tr - ac.od);
      pridej_cestu (tr + 1, ac.od + ac.len - tr - 1);
    }
  else if (nob == 2)
    should_play = tr;
}

int
main (int argc, const char *argv[])
{
  unsigned i, opt;
  int tah[2], tn;

  srand (0);

  const char *fn = (argc > 1) ? argv[2] : "gamesa.in";
  FILE *plan = fopen (fn, "r");
  if (!plan)
    {
      fprintf (stderr, "Popis hraciho planu nelze otevrit.\n");
      return 2;
    }

  log = fopen("gamesa.out", "w");

  fscanf (plan, "%d %d", &opt, &n);

  for (i = 0; i < n - 3; i++)
    fscanf (plan, "%d%d", &uhlopricky[i][0], &uhlopricky[i][1]);
  fclose (plan);

  herni_plan (n, uhlopricky);
  inicializace (uhlopricky);

  while (1)
    {
      tah_honzika (tah);
      fprintf (log, "H: %d %d\n", tah[0], tah[1]);

      hrana h(tah[0], tah[1]);
      if (!vceste.count (h))
	{
	  log_err ("Neexistujici hrana.");
	  return 1;
	}
      tn = vceste[h];
      if (obarveny[h])
	{
	  log_err ("Jiz obarvena hrana.");
	  return 1;
	}
      obarveny[h] = true;

      if (tn >= 0)
	{
	  if (test_honzik_vyhral (tn))
	    return 0;
	  zvys_obarvenost (tn);
	}
      else
	{
	  if (test_honzik_vyhral (-tn) || test_honzik_vyhral (-tn - 1))
	    return 0;
	  zvys_obarvenost (-tn);
	  zvys_obarvenost (-tn - 1);
	}

      if (should_play >= 0)
	{
	  zvitezit (should_play);
	  return 1;
	}

      if (opt)
	zvol_tah_marenky (tah);
      else
	nahodny_tah (tah);
      fprintf (log, "M: %d %d\n", tah[0], tah[1]);
      h = hrana (tah[0], tah[1]);
      obarveny[h] = true;
      tn = vceste[h];
      if (tn >= 0)
	zvys_obarvenost (tn);
      else
	{
	  zvys_obarvenost (-tn);
	  zvys_obarvenost (-tn - 1);
	}

      tah_marenky (tah);
    }

  return 0;
}
